// Display a sprite and do many things with the GsSprite Structure 
// This is a combined file with pad routines from Nelson Santos/ROGUE
//	and Sprite.c from Ira Rainey

// -----------------------------------------------
// Includes
// -----------------------------------------------

#include <libps.h>
#include "rougepad.h"				// Include the rougepad.h header file


// -----------------------------------------------
// Defines
// -----------------------------------------------

// Total number of sprites plus one
#define		SPRITE_CNT	(1+1)

// Screen size detail
#define		SCREEN_WIDTH	320
#define		SCREEN_HEIGHT	240

// Tim address
// Load your TIM file here
#define		spriteAdd	0x80100000
// Set ordering table length
#define OT_LENGTH	1


// -----------------------------------------------
// Function Prototypes
// -----------------------------------------------
void InitSprite ();
void SomePad ();


// -----------------------------------------------
// Initialise Structures
// -----------------------------------------------

// Ordering table variables & Global Vars
GsOT			WorldOT[2];
GsOT_TAG		OTTags[2][1<<OT_LENGTH];
int				activeBuff;
int Quit = 0;

// Primitive related vaiables
PACKET			GpuPacketArea[2][SPRITE_CNT*sizeof(GsSPRITE)];

// Sprite structure pointer
GsSPRITE		TestSprite;

void main(void)
{
	int	i;

	PadInit();					// First initialize the joypads
	SetVideoMode (MODE_PAL);


	// Initialise the drawing system
	GsInitGraph(SCREEN_WIDTH,SCREEN_HEIGHT,4,0,0);
	GsDefDispBuff(0,0,0,SCREEN_HEIGHT);
	ResetGraph(0);
		
	// Initialise the ordering table
	for (i=0;i<2;i++)
		{
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
		}

	// Initialise the sprite
	InitSprite();


	while (Quit == 0)
	{
		Pad1 = PadRead(1);		// Read in input data from Pad 1
		Pad2 = PadRead(2);		// Read in input data from Pad 2

		if (Pad1 != 0)
			{
				SomePad();
			}


		// Find out which buffer is being used
		activeBuff = GsGetActiveBuff();

		// Packet area
		GsSetWorkBase((PACKET *)GpuPacketArea[activeBuff]);

		// Clear the ordering table
		GsClearOt(0,0,&WorldOT[activeBuff]);

  		// Register sprite into ordering table
		// Uses GsSortSprite for rotation/scaling
		GsSortSprite(&TestSprite,&WorldOT[activeBuff],0);

		// Wait for all drawing to finish
		DrawSync(0);

		// Wait for vertical blank interrupt
		VSync(0);

		// Switch display buffer & offscreen buffer
		GsSwapDispBuff();

		// Sort the ordering table
		GsSortClear(0,0,0,&WorldOT[activeBuff]);

		// Draw the objects registered in the Ordering table
		GsDrawOt(&WorldOT[activeBuff]);
	}
}



//
// Sprite initialisation routine
//

void InitSprite()
	{
	RECT		rect;								// Define RECT structure
	GsIMAGE	tim_data;							// Define GsIMAGE structure
	
	// Get the TIM file info - skip 4 bytes from start of file
	// to get to info we need
	GsGetTimInfo ((u_long *)(spriteAdd+4),&tim_data);


	// Load the image into the frame buffer
	rect.x = tim_data.px;							// x pos in frame buffer
	rect.y = tim_data.py;							// y pos in frame buffer
	rect.w = tim_data.pw;							// width of image
	rect.h = tim_data.ph;							// height of image
	LoadImage(&rect, tim_data.pixel);				// load data into frame buffer

	// Load the CLUT into the frame buffer
	rect.x = tim_data.cx;							// x pos in frame buffer
	rect.y = tim_data.cy;							// y pos in frame buffer
	rect.w = tim_data.cw;							// width of CLUT
	rect.h = tim_data.ch;							// height of CLUT
	LoadImage(&rect, tim_data.clut);				// load data into frame buffer

	// Initialise sprite structure
	TestSprite.attribute = 0x01000000;						// sprite attribute - 8 bit
	TestSprite.x = ((SCREEN_WIDTH/2)-(tim_data.pw));		// sprite x axis position
	TestSprite.y = ((SCREEN_HEIGHT/2)-(tim_data.ph/2));// sprite y axis position
	
	// The width of an 8 bit sprite is half that of a 16 bit hence the *2
	TestSprite.w = (tim_data.pw*2);				// sprite width
	TestSprite.h = tim_data.ph;					// sprite height
	TestSprite.tpage = GetTPage(1,0, tim_data.px, tim_data.py);
	TestSprite.u = 0;							// Texture page offset x axis
	TestSprite.v = 0;							// Texture page offset y axis
	TestSprite.cx = tim_data.cx;					// CLUT x axis position
	TestSprite.cy = tim_data.cy;					// CLUT y axis position
	TestSprite.r = 128;							// Regular brightness
	TestSprite.g = 128;							// Regular brightness
	TestSprite.b = 128;							// Regular brightness
	TestSprite.mx = TestSprite.w / 2;				// Middle of rotation x
	TestSprite.my = TestSprite.h / 2;				// Middle of rotation y
	TestSprite.scalex = ONE;						// Defined as 4096
	TestSprite.scaley = ONE;						// Defined as 4096
	TestSprite.rotate = 0;						// Initial rotation

	// Wait until LoadImage() has finished before returning
	DrawSync(0);
	}



//
// Something happed to PAD1 Routine
//

void SomePad()
	{
			if ( (Pad1 & PADSTART) && (Pad1 & PADSELECT) )
				{
					printf("You pressed the Quit button combo on Pad 1. Bye!\n\n");
					Quit = 1;
				}
			
			if  (Pad1 & PADRIGHT)
				{
					TestSprite.x+=5;
				} 
			if  (Pad1 & PADLEFT)
				{
					TestSprite.x-=5;
				} 		
			if  (Pad1 & PADDOWN)
				{
					TestSprite.y+=5;
				} 
			if  (Pad1 & PADUP)
				{
					TestSprite.y-=5;
				} 

			// rotate sprite clockwise
			if (Pad1 & PADL1)
				{
					TestSprite.rotate+=(ONE * 10);
					if (TestSprite.rotate>1470464) TestSprite.rotate=0;
				}
	
			// rotate sprite anti-clockwise	
			if (Pad1 & PADR1)
				{
					TestSprite.rotate-=(ONE * 10);
					if (TestSprite.rotate<0) TestSprite.rotate=1470464;
				}
	
			// scale sprite down in size	
			if (Pad1 & PADL2) 
				{
					TestSprite.scalex=TestSprite.scaley-=100;
					if (TestSprite.scalex<101) TestSprite.scalex=TestSprite.scaley=101;
				}

			// scale sprite up in size
			if (Pad1 & PADR2)
				{
					TestSprite.scalex=TestSprite.scaley+=100;
					if (TestSprite.scaley>8096) TestSprite.scaley=TestSprite.scalex=8096;
				}
			// Change brightness
			if (Pad1 & PADSQUARE)
				{
					TestSprite.r=TestSprite.g=TestSprite.b=128;
				}

			// Change Red
			if (Pad1 & PADCIRCLE)
				{
					TestSprite.r+=5;
				}
			// Change Green
			if (Pad1 & PADTRIANGLE)
				{
					TestSprite.g+=5;
				}

			// Change Blueness
			if (Pad1 & PADX)
				{
					TestSprite.b+=5;
				}
	}
